// #pragma comment(linker, "/stack:200000000")
// #pragma GCC optimize("Ofast,no-stack-protector")
// #pragma GCC target("sse,sse2,sse3,ssse3,sse4,popcnt,abm,mmx,avx,avx2,tune=native")
// #pragma GCC optimize("unroll-loops")

#include <stdio.h>
#include <bits/stdc++.h>

#ifdef PERVEEVM_LOCAL
    #define debug(x) std::cerr << (#x) << ":\t" << (x) << std::endl
#else
    #define debug(x) 238
#endif

#define fastIO std::ios_base::sync_with_stdio(false); std::cin.tie(0); std::cout.tie(0)
#define NAME "File"

using ll = long long;
using ld = long double;

#ifdef PERVEEVM_LOCAL
    std::mt19937 rnd(238);
#else
    std::mt19937 rnd(std::chrono::high_resolution_clock::now().time_since_epoch().count());
#endif

template<typename T>
bool smin(T& a, const T& b) {
    if (b < a) {
        a = b;
        return true;
    }
    return false;
}

template<typename T>
bool smax(T& a, const T& b) {
    if (a < b) {
        a = b;
        return true;
    }
    return false;
}

const double PI = atan2(0.0, -1.0);
const int INF = 0x3f3f3f3f;
const ll LINF = (ll)2e18;
const int N = 1 << 19;

class Node {
public:
    std::pair<int, int> mx;
    int l, r;

    Node() : mx(std::make_pair(-INF, -1)), l(-1), r(-1) {}

    Node(std::pair<int, int> _mx, int _l, int _r) : mx(_mx), l(_l), r(_r) {}

    Node operator+(const Node& other) const {
        return Node(std::max(mx, other.mx), l, other.r);
    }
};

class SegmentTree {
public:
    void build() {
        for (int i = 0; i < N; ++i) {
            tree[i + N] = Node(std::make_pair(-INF, i), i, i + 1);
        }
        for (int i = N - 1; i > 0; --i) {
            tree[i] = tree[i * 2] + tree[i * 2 + 1];
        }
    }

    Node get(int v, int l, int r) {
        if (tree[v].r <= l || r <= tree[v].l) {
            return Node();
        }
        if (l <= tree[v].l && tree[v].r <= r) {
            return tree[v];
        }
        return get(v * 2, l, r) + get(v * 2 + 1, l, r);
    }

    void upd(int v, int pos, int val) {
        if (tree[v].l + 1 == tree[v].r) {
            smax(tree[v].mx.first, val);
            return;
        }

        if (pos < tree[v * 2].r) {
            upd(v * 2, pos, val);
        } else {
            upd(v * 2 + 1, pos, val);
        }

        tree[v] = tree[v * 2] + tree[v * 2 + 1];
    }

    Node tree[2 * N];
};

std::vector<int> values;
std::pair<int, int> a[N];
SegmentTree t;
int dpPref[N], dpSuff[N];

int transform(int x) {
    return std::lower_bound(values.begin(), values.end(), x) - values.begin();    
}

void run() {
    int n, x;
    scanf("%d%d", &n, &x);

    for (int i = 0; i < n; ++i) {
        scanf("%d%d", &a[i].first, &a[i].second);
        values.push_back(a[i].second);
        values.push_back(a[i].second - x);
        values.push_back(a[i].second + x);
        values.push_back(a[i].second + 2 * x);
    }

    std::sort(values.begin(), values.end());
    values.resize(std::unique(values.begin(), values.end()) - values.begin());
    
    std::sort(a, a + n, [&](const std::pair<int, int>& p1, const std::pair<int, int>& p2) {
        return p1.first < p2.first || (p1.first == p2.first && p1.second < p2.second);
    });

    t.build();
    for (int i = 0; i < n; ++i) {
        dpPref[i] = 1;
        int l = transform(a[i].second - x);
        int r = transform(a[i].second);
        smax(dpPref[i], 1 + t.get(1, l, r + 1).mx.first);
        t.upd(1, r, dpPref[i]);
    }

    t.build();
    for (int i = n - 1; i >= 0; --i) {
        dpSuff[i] = 1;
        int l = transform(a[i].second);
        int r = transform(a[i].second + x);
        smax(dpSuff[i], 1 + t.get(1, l, r + 1).mx.first);
        t.upd(1, l, dpSuff[i]);
    }

    t.build();
    int bestA = 0, bestB = 0;
    int bestAns = 0;
    for (int i = n - 1; i >= 0; --i) {
        int curAns = dpPref[i] + 1;

        int l = transform(a[i].second);
        int r = transform(a[i].second + 2 * x);
        auto res = t.get(1, l, r + 1).mx;
        curAns += std::max(0, res.first);

        if (smax(bestAns, curAns)) {
            bestA = a[i].first;
            if (res.first != -INF) {
                bestB = (a[i].second + values[res.second]) / 2;
            } else {
                bestB = a[i].second;
            }
        }

        t.upd(1, l, dpSuff[i]);
    }

    printf("%d %d %d\n", bestAns, bestA, bestB);
}

int main(void) {
    // freopen(NAME".in", "r", stdin);
    // freopen(NAME".out", "w", stdout);

    #ifdef PERVEEVM_LOCAL
        auto start = std::chrono::high_resolution_clock::now();
    #endif

    run();

    #ifdef PERVEEVM_LOCAL
        auto end = std::chrono::high_resolution_clock::now();
        std::cerr << "Execution time: "
                  << std::chrono::duration_cast<std::chrono::milliseconds>(end - start).count()
                  << " ms" << std::endl;
    #endif

    return 0;
}
